@@ -57,7 +57,8 @@ to display various salutiations.
 
 To explain this code, the *@web_api* decorator declares a method that is mapped to a route. The route is determined
 by the class name, in this case *Greetings*, and the method name. Thus, the "welcome" method above, as a member of
-*Greetings* class, is mapped to the route '/Greetings/welcome".  There are some rules about methods declared as *@web_api*:
+*Greetings* class, is mapped to the route '/Greetings/welcome".  There are some rules about methods declared
+as *@web_api*:
 
 #. They must be @staticmethod's.
 #. They must provide all type hints for parameters and return value,
@@ -239,7 +240,7 @@ class WebApplication:
         if html_out.exists():
             os.remove(html_out)
         css = """
-        
+
         h2{
             margin: 1em 0 .6em 0;
             font-weight: normal;
@@ -263,7 +264,6 @@ class WebApplication:
             letter-spacing: 1px;
             font-style: italic;font-size: 90%; background-color: #fed8b1;
         }
-        
         """
         css_dir: Path = self._static_path.joinpath('css')
         if not css_dir.exists():
@@ -304,10 +304,11 @@ class WebApplication:
         """
         if route in WebApplication.routes_get or route in cls.routes_post:
             existing = WebApplication.callables_get.get(route) or WebApplication.callables_post.get(route)
-            raise WebApplication.DuplicateRoute(
-                f"Route '{route}' associated with {api.module}.{api.name}"
-                f" already exists here: {existing.module}.{existing.name} "
-            )
+            if api.module != existing.module or api.name != existing.name:
+                raise WebApplication.DuplicateRoute(
+                    f"Route '{route}' associated with {api.module}.{api.name}"
+                    f" already exists here: {existing.module}.{existing.name} "
+                )
         cls.routes_get[route] = async_handler
         cls.callables_get[route] = api
 
@@ -429,12 +430,7 @@ class WebApplication:
                 instance = clazz_(*args, **kargs)
                 if hasattr(clazz_, '__aenter__'):
                     await instance.__aenter__()
-                self_id = self_id or hex(id(instance))
-                cls.ObjectRepo.instances[self_id] = instance
-                cls.ObjectRepo.by_instance[instance] = self_id
-                cls.ObjectRepo.expiry[self_id] = asyncio.create_task(cls.ObjectRepo.expire_obj(
-                    self_id, cls.ObjectRepo.DEFAULT_OBJECT_EXPIRATION))
-                return self_id
+                return instance
 
             clazz_._create = _create
             if hasattr(clazz_, '__init__') and hasattr(clazz_.__init__, '__annotations__'):
@@ -444,16 +440,16 @@ class WebApplication:
             clazz_._create.__annotations__['return'] = str
 
             clazz_._create.__doc__ = clazz_.__init__.__doc__ if hasattr(clazz_, '__init__') else f"""
-                Create an instance of {clazz_.__name__} on server, returning a unique string id for the instnace.  
-                The instance will remain active until /{clazz_.__name__}/expire is invoked.  The sting is used for 
+                Create an instance of {clazz_.__name__} on server, returning a unique string id for the instance.
+                The instance will remain active until /{clazz_.__name__}/expire is invoked.  The sting is used for
                 instance-based ReST methods to act on the created instance on the server.
 
                 :return: unqiue string id of instance created
                 """
             clazz_._create.__qualname__ = f"{clazz_.__name__}._create"
             # noinspection PyProtectedMember
-            cls._func_wrapper(clazz, clazz_._create, is_instance_method=False, is_constructor=True, expire_on_exit=False,
-                              method=RestMethod.GET, content_type="text/plain")
+            cls._func_wrapper(clazz, clazz_._create, is_instance_method=False, is_constructor=True,
+                              expire_on_exit=False, method=RestMethod.GET, content_type="text/plain")
 
             async def _expire(self, new_lease_time: int = cls.ObjectRepo.DEFAULT_OBJECT_EXPIRATION,
                               _uuid: Optional[str] = None) -> None:
@@ -616,7 +612,7 @@ class WebApplication:
                 kwargs.update(addl_args)
             if api.is_instance_method:
                 # we are invoking a class method, and need to lookup instance
-                self_id = kwargs.get('self')
+                self_id = kwargs.get('self').replace('\\', '')
                 if self_id is None:
                     raise ValueError(
                         "A self request parameter is needed. No instance provided for call to instance method")
@@ -624,14 +620,10 @@ class WebApplication:
                 if instance is None:
                     raise ValueError(f"No instance found for request with 'self' id of {self_id}")
                 del kwargs['self']
-                result = await api(instance, **kwargs)
+                result = api(instance, **kwargs)
             else:
                 # call the underlying function:
-                result = await api(**kwargs)
-                if api.is_constructor:
-                    uuid = kwargs.get(api.uuid_param) or hex(id(result))
-                    cls.ObjectRepo.instances[uuid] = result
-                    cls.ObjectRepo.by_instance[result] = uuid
+                result = api(**kwargs)
             if inspect.isasyncgen(result):
                 #################
                 #  streamed response through async generator:
@@ -657,11 +649,23 @@ class WebApplication:
                 #################
                 #  regular response
                 #################
+                result = await result
+                instance = result
                 if api.is_constructor:
-                    try:
-                        result = cls.ObjectRepo.by_instance[result]
-                    except KeyError:
-                        raise SyntaxError("No uuid set for created instance")
+                    if api.clazz and hasattr(api.clazz, '__aenter__'):
+                        await instance.__aenter__()
+                    if hasattr(api.clazz, 'jsonrepr'):
+                        repr = api.clazz.jsonrepr(result)
+                        uuid = repr.get(api.uuid_param)
+                        content_type = 'application/json'
+                        result = json.dumps(repr)
+                    else:
+                        uuid = kwargs.get(api.uuid_param) or hex(id(result))
+                        result = uuid
+                    cls.ObjectRepo.instances[uuid] = instance
+                    cls.ObjectRepo.by_instance[instance] = uuid
+                    cls.ObjectRepo.expiry[uuid] = asyncio.create_task(cls.ObjectRepo.expire_obj(
+                        uuid, cls.ObjectRepo.DEFAULT_OBJECT_EXPIRATION))
                 else:
                     result = _serialize_return_value(result, encoding)
                 return Response(status=200, body=result if result is not None else b"Success",
